@@ -157,6 +157,7 @@ ospf_ase_forward_address_check (struct ospf *ospf, struct in_addr fwd_addr)
   return 1;
 }
 
+#if 0
 /* Calculate ASBR route. */
 static struct ospf_route *
 ospf_ase_calculate_asbr_route (struct ospf *ospf,
@@ -237,6 +238,7 @@ ospf_ase_calculate_asbr_route (struct ospf *ospf,
 
   return asbr_route;
 }
+#endif
 
 static struct ospf_route *
 ospf_ase_calculate_new_route (struct ospf_lsa *lsa,
