@@ -239,6 +239,7 @@ rip_request_interface (struct interface *ifp)
   }
 }
 
+#if 0
 /* Send RIP request to the neighbor. */
 static void
 rip_request_neighbor (struct in_addr addr)
@@ -269,6 +270,7 @@ rip_request_neighbor_all (void)
     if (rp->info)
       rip_request_neighbor (rp->p.u.prefix4);
 }
+#endif
 
 /* Multicast packet receive socket. */
 static int
@@ -423,8 +425,9 @@ rip_interface_up (int command, struct zclient *zclient, zebra_size_t length)
     return 0;
 
   if (IS_RIP_DEBUG_ZEBRA)
-    zlog_debug ("interface %s index %d flags %lld metric %d mtu %d is up",
-	       ifp->name, ifp->ifindex, ifp->flags, ifp->metric, ifp->mtu);
+    zlog_debug ("interface %s index %d flags %#llx metric %d mtu %d is up",
+	       ifp->name, ifp->ifindex, (unsigned long long) ifp->flags,
+	       ifp->metric, ifp->mtu);
 
   /* Check if this interface is RIP enabled or not.*/
   rip_enable_apply (ifp);
@@ -447,8 +450,9 @@ rip_interface_add (int command, struct zclient *zclient, zebra_size_t length)
   ifp = zebra_interface_add_read (zclient->ibuf);
 
   if (IS_RIP_DEBUG_ZEBRA)
-    zlog_debug ("interface add %s index %d flags %lld metric %d mtu %d",
-	       ifp->name, ifp->ifindex, ifp->flags, ifp->metric, ifp->mtu);
+    zlog_debug ("interface add %s index %d flags %#llx metric %d mtu %d",
+		ifp->name, ifp->ifindex, (unsigned long long) ifp->flags,
+		ifp->metric, ifp->mtu);
 
   /* Check if this interface is RIP enabled or not.*/
   rip_enable_apply (ifp);
@@ -486,8 +490,9 @@ rip_interface_delete (int command, struct zclient *zclient,
     rip_if_down(ifp);
   } 
   
-  zlog_info("interface delete %s index %d flags %lld metric %d mtu %d",
-	    ifp->name, ifp->ifindex, ifp->flags, ifp->metric, ifp->mtu);  
+  zlog_info("interface delete %s index %d flags %#llx metric %d mtu %d",
+	    ifp->name, ifp->ifindex, (unsigned long long) ifp->flags,
+	    ifp->metric, ifp->mtu);
   
   /* To support pseudo interface do not free interface structure.  */
   /* if_delete(ifp); */
@@ -947,8 +952,6 @@ rip_interface_wakeup (struct thread *t)
   return 0;
 }
 
-int rip_redistribute_check (int);
-
 static void
 rip_connect_set (struct interface *ifp, int set)
 {
